@@ -99,7 +99,7 @@ if (!$readonly) {
 		->addClass(ZBX_STYLE_BTN_GREY)
 		->onClick('return PopUp("popup.php?srctbl=items&srcfld1=itemid&dstfld1=master_itemid&dstfrm='.
 			$itemForm->getName().'&srcfld2=master_itemname&dstfld2=master_itemname&only_hostid='.$data['hostid'].
-			'&with_webitems=0&excludeids[]='.$data['itemid'].'");'
+			'&with_webitems=1&excludeids[]='.$data['itemid'].'");'
 		);
 }
 
@@ -463,9 +463,14 @@ else {
 		$valuemapComboBox->addItem($valuemap['valuemapid'], CHtml::encode($valuemap['name']));
 	}
 }
-$link = (new CLink(_('show value mappings'), 'adm.valuemapping.php'))
-	->setAttribute('target', '_blank');
-$itemFormList->addRow(_('Show value'), [$valuemapComboBox, SPACE, $link], 'row_valuemap');
+
+if (CWebUser::getType() == USER_TYPE_SUPER_ADMIN) {
+	$valuemapComboBox = [$valuemapComboBox, '&nbsp;',
+		(new CLink(_('show value mappings'), 'adm.valuemapping.php'))->setAttribute('target', '_blank')
+	];
+}
+$itemFormList->addRow(_('Show value'), $valuemapComboBox, 'row_valuemap');
+
 $itemFormList->addRow(_('Allowed hosts'),
 	(new CTextBox('trapper_hosts', $data['trapper_hosts'], $discovered_item))->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH),
 	'row_trapper_hosts');
